<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00001001DocumentoPropostaService;
use Teknisa\SM\Service\Sm00001000DocumentoPropostaService as DocumentoService;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\Libs\Util\NewCode;

class Sm00001001DocumentoPropostaController extends SMCrud {
    
    protected $dataSourceName = 'fct_orcamentodoc_widget';
    protected $serviceDocumentoProposta;
    protected $documentoService;
    protected $zhFileUploader;
    protected $newCode;
    
    public function __construct($crudService, $environment, Sm00001001DocumentoPropostaService $serviceDocumentoProposta, DocumentoService $documentoService, Server $zhFileUploader, NewCode $newCode) {
        parent::__construct($crudService, $environment);
        $this->serviceDocumentoProposta = $serviceDocumentoProposta;
        $this->documentoService         = $documentoService;
        $this->zhFileUploader           = $zhFileUploader;
        $this->newCode                  = $newCode;
    }
    
    public function save(Request $request, Response $response) {
        try {
            $row      = $request->getDataSet()->getRows()[0];
            $nrorg    = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            if (isset($row["DSCAMINHOARQ"]) && is_array($row["DSCAMINHOARQ"])) {
                $row["NMARQUIVO"]    = $row["DSCAMINHOARQ"][0]["name"];
                $row["VRTAMANHO"]    = round($row["DSCAMINHOARQ"][0]["size"] / 1024, 2);
                $row["DSTIPO"]       = ($row["DSCAMINHOARQ"][0]["type"]) ? $row["DSCAMINHOARQ"][0]["type"] : 'application/vnd.openxmlformats-officedocument.wordprocessingml.document';
                $row["DSCAMINHOARQ"] = $row["DSCAMINHOARQ"][0]["path"];
            }
            
            if (!$row["__is_new"]) {
                $this->serviceDocumentoProposta->salvarDocumentoProposta($row, $nrorg, $operador);
            }
            else {
                //$this->serviceDocumentoProposta->validarSituacaoDocumento($row["IDSITUACAOORCAMENTO"], $row["NRSEQORCAMENTO"], $this->environment->getNrorg());
                $idSituacoes = $row["IDSITUACAOORCAMENTO"];
                foreach($idSituacoes as $idSituacao) {
                    $existe = $this->serviceDocumentoProposta->verificaSeExisteDocumentoPorPropostaESituacao($row["NRSEQORCAMENTO"], $idSituacao, $nrorg);
                    if (!$existe) {
                        $nrseqorcadoc = $this->newCode->getCodeByTableName("FCT_ORCAMENTODOC", $nrorg, 1, 10, true);
                        $row["IDSITUACAOORCAMENTO"] = $idSituacao;
                        $row["NRSEQORCADOC"]        = $nrseqorcadoc;
                        parent::save($request, $response);
                    }
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Atualiza o documento de acordo com o modelo encontrado.
     */
    public function atualizaDocumento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrSeqOrcaDoc   = $row["NRSEQORCADOC"];
            $nrSeqModDocFct = $row["NRSEQMODDOCFCT"];
    
            $this->serviceDocumentoProposta->atualizaDocumento($nrSeqOrcaDoc, $nrSeqModDocFct);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }    
    
     /**
     * Gera o Documento da Proposta.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function gerarDocumentoProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $nrorg           = $row["NRORG"];
            $idSituacao      = $row["IDSITUACAOORCAMENTO"];
            
            $this->documentoService->gerarDocumentoProposta($nrseqorcamento, $nrtipoorcamento, $nrorg, $idSituacao, "P");
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function excluirAnexo(Request $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqorcadoc = $row["NRSEQORCADOC"];
            $dscaminhoarq = $row["DSCAMINHOARQ"];
            $nrorg        = $row["NRORG"];
            
            $this->zhFileUploader->deleteFile($dscaminhoarq);
            $this->serviceDocumentoProposta->limparAnexoDaProposta($nrseqorcadoc, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Deleta os documentos de uma proposta
    **/
    public function deletarDocumentoProposta(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $nrseqorcadocs = explode("_", $row["NRSEQORCADOC"]); 
            $dscaminhoarq  = $row["DSCAMINHOARQ"];
            foreach($nrseqorcadocs as $nrseqorcadoc) {
                $row["NRSEQORCADOC"] = $nrseqorcadoc;
                parent::delete($request, $response);
            }
            
            if (!empty($dscaminhoarq))
                $this->zhFileUploader->deleteFile($dscaminhoarq);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
   
    /**
     * Efetua o download do documento no fileServer e salva na pasta tmp
    **/
    public function downloadDocFileServer(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqorcadoc = $row["NRSEQORCADOC"];
            $nrorg = $this->environment->getNrOrg();
        
            $result = $this->serviceDocumentoProposta->downloadDocFileServer($nrseqorcadoc, $nrorg);
            $response->addDataSet(new DataSet("URLDOCUMENTO", array($result)));   
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

?>